#include "tuya_cloud_types.h"
#include "uni_log.h"
#include "tuya_hal_bt.h"
#include "tuya_driver.h"
#include "tuya_hal_semaphore.h"
#include "tuya_hal_system.h"
#include "tuya_hal_thread.h"
#include "tuya_ws_db.h"
#include "bt_ext_porting.h"
#include "bt_uart_drv.h"
#include "mem_pool.h"
#include "string.h"
#include "sys_timer.h"
#include "tuya_os_adapter.h"


enum recv_proc {
    UART_RECV = 0,
    UART_PROC,
};

enum send_state {
    SEND_IDLE = 0,
    SEND_BUSY,
};

STATIC TY_EX_BT_MSG_S       s_btext_msg = {0};
STATIC TY_BT_MSG_CB         bt_msg_cb = NULL;
STATIC ty_bt_scan_info_t    *ble_scan_info = NULL;
STATIC TUYA_EXT_BT_GPIO_IQR_CB g_gpio_irq_cb = NULL;
STATIC TUYA_QUERY_HID_RSSI_CB g_query_rssi_cb = NULL;
STATIC MUTEX_HANDLE         g_bt_ext_mute = NULL;
STATIC TIMER_ID             g_query_rssi_timer = 0;
STATIC BOOL_T               g_hid_bond = FALSE;
STATIC BOOL_T               g_inited = FALSE;

STATIC OPERATE_RET tuya_start_query_rssi(VOID);
STATIC OPERATE_RET tuya_stop_query_rssi(VOID);


STATIC OPERATE_RET tuya_ext_bt_send(BYTE_T *data, UINT8_T len);
STATIC OPERATE_RET tuya_ext_bt_port_init(ty_bt_param_t *p);
STATIC OPERATE_RET tuya_ext_bt_port_deinit(void);
STATIC OPERATE_RET tuya_ext_bt_gap_disconnect(void);
STATIC OPERATE_RET tuya_ext_bt_reset_adv(tuya_ble_data_buf_t *adv, tuya_ble_data_buf_t *scan_resp);
STATIC OPERATE_RET tuya_ext_bt_start_adv(void);
STATIC OPERATE_RET tuya_ext_bt_stop_adv(void);
STATIC OPERATE_RET tuya_ext_bt_assign_scan(ty_bt_scan_info_t *info);
STATIC OPERATE_RET tuya_ext_bt_get_rssi(signed char *rssi);
STATIC OPERATE_RET tuya_ext_bt_start_scan();
STATIC OPERATE_RET tuya_ext_bt_stop_scan();
STATIC OPERATE_RET tuya_ext_bt_setmac(CONST NW_MAC_S *mac);
STATIC OPERATE_RET tuya_ext_bt_getmac(NW_MAC_S *mac);


STATIC TUYA_OS_BT_INTF m_tuya_os_bt_intfs = {
        .port_init      = tuya_ext_bt_port_init,
        .port_deinit    = tuya_ext_bt_port_deinit,
        .gap_disconnect = tuya_ext_bt_gap_disconnect,
        .send           = tuya_ext_bt_send,
        .reset_adv      = tuya_ext_bt_reset_adv,
        .get_rssi       = tuya_ext_bt_get_rssi,
        .start_adv      = tuya_ext_bt_start_adv,
        .stop_adv       = tuya_ext_bt_stop_adv,
        .assign_scan    = tuya_ext_bt_assign_scan,
        .scan_init      = NULL,
        .start_scan     = tuya_ext_bt_start_scan,
        .stop_scan      = tuya_ext_bt_stop_scan,
        .set_mac        = tuya_ext_bt_setmac,
        .get_mac        = tuya_ext_bt_getmac,
};

VOID __print_raw_bytes(BYTE_T* data, INT_T len)
{
    INT_T i = 0;
    for(;i < len;i++){
        printf("%x ", data[i]);
    }
    printf("\n");
}

int JKCom_check_code(unsigned char *recv_buf,unsigned int len)
{
    int j;
    unsigned int datasum = 0;
    unsigned char checksum;

    /* check code */
    for (j = 0;j < len; j++)
    {
        datasum += recv_buf[j];
    }

    checksum = datasum % 256;
    return checksum;
}


VOID __pack_bt_frame(BYTE_T* in_data, INT_T in_len, BYTE_T* out_data, INT_T* out_len)
{
    if (*out_len < in_len + 8)
    {
        printf("out_len too small(%d)\n", *out_len);
        return;
    }

    static BYTE_T seq = 1;

    *out_len = in_len + 8;
    out_data[0] = 0x55;
    out_data[1] = 0xAA;
    out_data[2] = 0x03;
    out_data[3] = 0x13;
    out_data[4] = (1+in_len) >> 8;
    out_data[5] = (1+in_len) & 0xFF;
    out_data[6] = seq++;
    memcpy(out_data+7, in_data, in_len);
    out_data[7+in_len] = JKCom_check_code(out_data, 7+in_len);
}

STATIC NW_MAC_S s_default_mac_addr = {
    .mac = {0x6F,0xDC,0xAB,0xAC,0xDD,0xC7}
};

/**
 * @brief 互斥锁
 *
 * @return STATIC
 */
STATIC VOID tuya_bt_ext_lock(VOID)
{
    if (!g_bt_ext_mute) {
        return;
    }
    tuya_hal_mutex_lock(g_bt_ext_mute);
}

/**
 * @brief 互斥锁
 *
 * @return STATIC
 */
STATIC VOID tuya_bt_ext_unlock(VOID)
{
    if (!g_bt_ext_mute) {
        return;
    }
    tuya_hal_mutex_unlock(g_bt_ext_mute);
}

/*串口接收*/
STATIC VOID ty_recv_proc(VOID)
{
    UINT_T count = tuya_bt_uart_read(s_btext_msg.proc_buf + s_btext_msg.rcount,
                                  HN_RECV_BUF_MAX - s_btext_msg.rcount);
    PR_DEBUG("read %d bytes", count);
    __print_raw_bytes(s_btext_msg.proc_buf+s_btext_msg.rcount, count);
    s_btext_msg.rcount += count;
    if( s_btext_msg.rcount >= HN_RECV_MIN_NUM ) {
        s_btext_msg.recv_state = UART_PROC;
    }

}

STATIC UCHAR_T getCheckSum(CONST UCHAR_T *pack, INT_T pack_len)
{
    UINT_T check_sum = 0;
    while(--pack_len >= 0)
    {
        check_sum += *pack++;
    }
    return check_sum&0xff;
}

void* my_memmove(void* dst, const void* src, size_t n)
{
    if (NULL == dst || NULL == src)
        return NULL;
    void* ret = dst;
    char *dst_=(char*)dst;
    char *src_=(char*)src;
    if (dst <= src ||dst_ >= (src_ + n))  //第一种和第二种的情况
    {
        while (n--)                       //正向进行复制（从头至尾）
        {
          *dst_++=*src_++;   
        }
    }
    else                                  //第三种情况             反向进行复制（从尾至头）
    {
        dst_ += n - 1;
        src_ += n - 1;
        while (n--)
        {
           *dst_ -- =*src_ --;
        }
    }
    return ret;
}

STATIC int my_strlen(const char* src)
{
    int len = 0;

    while(*src++ != '\0')
        len ++;
    return len;
}

STATIC INT_T ty_btext_upload_proc(UCHAR_T* data)
{

    BTUART_FRAME_S *ty_frame = (BTUART_FRAME_S *)data;
    ty_frame->len = WORD_SWAP(ty_frame->len);

    TUYA_BTUART_INFO("recv fr_head[0x%x] fr_type[0x%x] fr_len[0x%x]",
                     ty_frame->head, ty_frame->fr_type, ty_frame->len);

    INT_T i = 0;
    for (; i < ty_frame->len; i++)
    {
        TUYA_BTUART_INFO("recv data[%d]: 0x%x", i, ty_frame->data[i]);
    }

    if (bt_msg_cb)
    {
        tuya_ble_data_buf_t databuf;
        databuf.data = ty_frame->data + 1;
        databuf.len = ty_frame->len - 1;
        bt_msg_cb(0, TY_BT_EVENT_RX_DATA, &databuf);
    }

    return OPRT_OK;
}

/*消息处理*/
STATIC VOID ty_msg_proc(VOID)
{
    INT_T fr_len = 0, offset = 0;
    UCHAR_T check_num = 0;
    while((s_btext_msg.rcount - offset) >= HN_RECV_MIN_NUM) {
        if(s_btext_msg.proc_buf[offset] != 0x55) {
            offset += 1;
            // TUYA_BTUART_DEBUG("[0x55]head offset is ....:%d",offset);
            continue;
        }
        if(s_btext_msg.proc_buf[offset + 1] != 0xaa) {
            offset += 1;
            // TUYA_BTUART_DEBUG("[0xaa]head offset is ....:%d",offset);
            continue;
        }

        fr_len = s_btext_msg.proc_buf[offset + 4]*256 + s_btext_msg.proc_buf[offset + 5] + 6 + 1;
        if(fr_len > HN_RECV_BUF_MAX) {
            offset += 1;
            TUYA_BTUART_INFO("len offset is ....:%d",offset);
            continue;
        }

        if(s_btext_msg.rcount - offset < fr_len) {
            // TUYA_BTUART_DEBUG("ty_msg.rcount:%d,offset:%d,fr_len:%d",s_btext_msg.rcount,offset,fr_len);
            s_btext_msg.timer_out_cnt++;
            tuya_hal_system_sleep(20);
            if(s_btext_msg.timer_out_cnt > 32){
                TUYA_BTUART_INFO("pack is not over");
                s_btext_msg.timer_out_cnt = 0;
                offset += 2;
            }
            break;
        }
        s_btext_msg.timer_out_cnt = 0;

        check_num = getCheckSum(s_btext_msg.proc_buf + offset, fr_len - 1);
        if(check_num != s_btext_msg.proc_buf[offset + fr_len - 1]) {
            offset += 1;
            TUYA_BTUART_INFO("crc offset is ....:%d",offset);
            continue;
        }

        ty_btext_upload_proc(s_btext_msg.proc_buf + offset);
        offset += fr_len;
    }

    // TUYA_BTUART_INFO("offset is ....:%d",offset);
    if( offset > 0 ) {
        s_btext_msg.rcount -= offset;
        if(s_btext_msg.rcount > 0) {
            my_memmove(s_btext_msg.proc_buf, s_btext_msg.proc_buf + offset, s_btext_msg.rcount);
        }
    }
    s_btext_msg.recv_state = UART_RECV;
}

STATIC VOID ty_btext_thread(PVOID_T pArg)
{
    if(s_btext_msg.proc_buf == NULL) {
        s_btext_msg.proc_buf = (UCHAR_T *)Malloc(HN_RECV_BUF_MAX);
    }
    if (!s_btext_msg.proc_buf) {
        TUYA_BTUART_ERR("Malloc Fail!");
        goto BTUART_THREAD_FAIL;
    }
    s_btext_msg.recv_state = UART_RECV;
    s_btext_msg.rcount = 0;
    while(s_btext_msg.recv_thrad_running) {

        switch(s_btext_msg.recv_state) {
            case UART_RECV:
                ty_recv_proc();
                break;
            case UART_PROC:
                ty_msg_proc();
                break;
            default:
                TUYA_BTUART_INFO("########ty_btext_thread default");
                break;
        }
    }
BTUART_THREAD_FAIL:
    s_btext_msg.thread = NULL;
    TUYA_BTUART_ERR("%s deleted",__func__);
    tuya_hal_thread_release(NULL);
}


STATIC OPERATE_RET tuya_ext_bt_send(BYTE_T *data, UINT8_T len)
{
    OPERATE_RET op_ret = OPRT_OK;
    RECV_FRAME_S recv = {0};

    if (data == NULL || len == 0) {
        return OPRT_INVALID_PARM;
    }
    
    INT_T i = 0;
    TUYA_BTUART_DEBUG("BT TX:");
    __print_raw_bytes(data, len);
    op_ret = tuya_bt_uart_write(data,len);
    if (op_ret != OPRT_OK) {
        return OPRT_OS_ADAPTER_BT_SEND_FAILED;
    }
    return op_ret;
}

STATIC OPERATE_RET tuya_ext_bt_port_init(ty_bt_param_t *p)
{
    OPERATE_RET op_ret = OPRT_OK;
    
    TUYA_BTUART_INFO("tuya ext bt port init");

    tuya_ext_bt_uart_init(0, 115200);

    s_btext_msg.recv_thrad_running = TRUE;
    if (!s_btext_msg.thread) {
        op_ret = tuya_hal_thread_create(&s_btext_msg.thread,
                            "ty_btext_task",
                            TUYA_GPS_KERNAL_HEAP_SIZE,
                            TUYA_GPS_KERNAL_PRIORITY,
                            ty_btext_thread,NULL);

        if (op_ret != OPRT_OK) {
            TUYA_BTUART_ERR("thread ty_btext_thread err");
            return op_ret;
        } else {
            TUYA_BTUART_INFO("thread ty_btext_thread create cplt");
        }
    }
    
    if (p) {
        bt_msg_cb  = p->cb;
    }

    return op_ret;

}

STATIC OPERATE_RET tuya_ext_bt_port_deinit(void)
{
    s_btext_msg.recv_thrad_running = FALSE;
    return OPRT_OS_ADAPTER_OK;
}

/**
 * @brief 用于断开蓝牙连接
 *
 * @return int 0=成功，非0=失败
 */
STATIC OPERATE_RET tuya_ext_bt_gap_disconnect(void)
{
    RECV_FRAME_S recv = {0};
    OPERATE_RET ret = OPRT_OK;
    
    return ret;
}

/**
 * @brief 用于重置蓝牙广播内容
 *
 * @param[in]       adv
  * @param[in]      scan_resp
 * @return int 0=成功，非0=失败
 */
STATIC OPERATE_RET tuya_ext_bt_reset_adv(tuya_ble_data_buf_t *adv, tuya_ble_data_buf_t *scan_resp)
{
    TUYA_BTUART_INFO("reset adv: ");
    __print_raw_bytes(adv->data, adv->len);
    TUYA_BTUART_INFO("resp: ");
    __print_raw_bytes(scan_resp->data, scan_resp->len);

    BYTE_T send_data[256] = {0};
    INT_T send_len = 256;

    __pack_bt_frame(adv->data, adv->len, send_data, &send_len);
    OPERATE_RET op_ret = tuya_bt_uart_write(send_data, send_len);
    if (op_ret != OPRT_OK)
    {
        return OPRT_OS_ADAPTER_BT_SEND_FAILED;
    }
    return OPRT_OS_ADAPTER_OK;
}

/**
 * @brief 用于启动蓝牙广播
 *
 * @return int 0=成功，非0=失败
 */
STATIC OPERATE_RET tuya_ext_bt_start_adv(void)
{
    TUYA_BTUART_INFO("start adv");
    return OPRT_OK;
}

/**
 * @brief 用于停止蓝牙广播
 *
 * @return int 0=成功，非0=失败
 */
STATIC OPERATE_RET tuya_ext_bt_stop_adv(void)
{
    TUYA_BTUART_INFO("stop adv");
    return OPRT_OK;
}

/**
 * @brief 用于扫描蓝牙信标(厂测使用)
 *
 * @param[out]       rssi
 * @return int 0=成功，非0=失败
 */
STATIC OPERATE_RET tuya_ext_bt_assign_scan(ty_bt_scan_info_t *info)
{
    return OPRT_OK;
}

STATIC OPERATE_RET tuya_ext_bt_start_scan()
{
    return OPRT_OS_ADAPTER_BT_SCAN_FAILED;
}

STATIC OPERATE_RET tuya_ext_bt_stop_scan()
{
    return OPRT_OS_ADAPTER_BT_SCAN_FAILED;
}

/**
 * @brief 用于获取蓝牙信号强度
 *
 * @param[out]       rssi
 * @return int 0=成功，非0=失败
 */
STATIC OPERATE_RET tuya_ext_bt_get_rssi(signed char *rssi)
{
    return OPRT_OK;
}

STATIC OPERATE_RET tuya_ext_bt_setmac(CONST NW_MAC_S *mac)
{
    OPERATE_RET ret = OPRT_OK;

    if (mac == NULL) {
        return OPRT_OS_ADAPTER_INVALID_PARM;
    }

    ret = wd_common_write(BT_MAC_ADDR_PARAM, mac->mac, TY_MAC_ADDR_LEN);
    if (ret < 0) {
        return OPRT_OS_ADAPTER_BT_INIT_FAILED;
    }

    TUYA_BTUART_INFO("tuya_ext_bt_setmac %x:%x:%x:%x:%x:%x",
             mac->mac[0], mac->mac[1], mac->mac[2], mac->mac[3],
             mac->mac[4], mac->mac[5]);

    return OPRT_OS_ADAPTER_OK;
}

STATIC OPERATE_RET tuya_ext_bt_getmac(NW_MAC_S *mac)
{
    OPERATE_RET ret = OPRT_OK;
    BYTE_T *p_mac = NULL;
    UINT_T len = 0;

    if (mac == NULL) {
        return OPRT_OS_ADAPTER_INVALID_PARM;
    }

    ret = wd_common_read(BT_MAC_ADDR_PARAM, &p_mac, &len);
    if (ret < 0) {
        return OPRT_OS_ADAPTER_BT_INIT_FAILED;
    }

    memcpy(mac->mac, p_mac, TY_MAC_ADDR_LEN);
    TUYA_BTUART_INFO("tuya_ext_bt_getmac %x:%x:%x:%x:%x:%x",
             mac->mac[0], mac->mac[1], mac->mac[2], mac->mac[3],
             mac->mac[4], mac->mac[5]);

    return OPRT_OS_ADAPTER_OK;
}

OPERATE_RET tuya_ext_bt_init(VOID)
{
   return tuya_os_adapt_reg_intf(INTF_BT, &m_tuya_os_bt_intfs);
}

OPERATE_RET tuya_ext_bt_uart_init(tuya_uart_port_t port,INT_T baudrate)
{
    OPERATE_RET op_ret = OPRT_OK;
    if (g_inited == TRUE){
        TUYA_BTUART_NOTICE("already inited");
        return op_ret;
    }

    op_ret = tuya_hal_mutex_create_init(&g_bt_ext_mute);
    if (op_ret != OPRT_OK) {
        TUYA_BTUART_ERR("ext bt mutex create fail");
        return op_ret;
    }

    op_ret = tuya_bt_uart_open(port, baudrate);
    if (op_ret != OPRT_OK) {
        TUYA_BTUART_INFO("tuya bt uart open fail");
        return op_ret;
    }

    op_ret = tuya_ext_bt_init();
    if (op_ret != OPRT_OK) {
        TUYA_BTUART_INFO("tuya ext bt init fail");
        return op_ret;
    }

#if 1
    tuya_os_bt_intf_init();
#else
    /* 测试BLE指令 */
    tuya_ext_bt_port_init(NULL);
#endif
    g_inited = TRUE;

    return op_ret;
}
