#include "tuya_cloud_types.h"
#include "uni_log.h"
#include "tuya_driver.h"
#include "tuya_uart.h"
#include "bt_ext_porting.h"
#include "bt_uart_drv.h"
#include "mem_pool.h"

STATIC struct tuya_bt_uart_drv *bt_drv = NULL;

OPERATE_RET tuya_bt_uart_init(tuya_uart_port_t port, tuya_uart_cfg_t *uart_cfg);

OPERATE_RET tuya_bt_uart_write(UCHAR_T *data,UINT16_T len) ;

INT_T tuya_bt_uart_read(UCHAR_T *data, UINT16_T len);

INT_T tuya_gps_bt_uart_timeout(UCHAR_T *data, UINT16_T len,UINT16_T timeout);


/**
 * @brief 打印设备配置
 *
 * @return STATIC
 */
STATIC VOID tuya_bt_uart_print_info(tuya_uart_t *uart)
{
    if (uart) {
        TUYA_BTUART_INFO("uart bandrate:%d",uart->cfg.baudrate);
        TUYA_BTUART_INFO("uart parity:%d", uart->cfg.parity);
        TUYA_BTUART_INFO("uart stopbits:%d", uart->cfg.stopbits);
        TUYA_BTUART_INFO("uart bufsz:%d", uart->cfg.bufsz);
    }
}



OPERATE_RET tuya_bt_uart_init(tuya_uart_port_t port, tuya_uart_cfg_t *uart_cfg)
{
    OPERATE_RET op_ret = OPRT_OK;

    if (uart_cfg == NULL) {
        return OPRT_INVALID_PARM;
    }
    if (bt_drv) {
        TUYA_BTUART_INFO("drv have inited,reinit uart%d!!!",port);
    } else {
        /* 获取管理句柄 */
        bt_drv = (struct tuya_bt_uart_drv *)Malloc(SIZEOF(struct tuya_bt_uart_drv));
        if (!bt_drv) {
            TUYA_BTUART_ERR("uart malloc fail!!!");
            return OPRT_MALLOC_FAILED;
        }
        op_ret = tuya_hal_mutex_create_init(&bt_drv->mutex);
        if (op_ret != OPRT_OK) {
            TUYA_BTUART_ERR("create mutex err");
            return op_ret;
        }
    }
    bt_drv->bt_uart_port = port;

    /* 查找对应的驱动 */
    bt_drv->bt_uart = tuya_driver_find(TUYA_DRV_UART, port);
    if (NULL == bt_drv->bt_uart)
    {
        TUYA_BTUART_ERR("uart%d not find!!!",port);
        return OPRT_COM_ERROR;
    }

    TUYA_UART_8N1_CFG(bt_drv->bt_uart, uart_cfg->baudrate, uart_cfg->bufsz, 0);

    /* 配置驱动 */
    op_ret = tuya_uart_init(bt_drv->bt_uart);
    if (OPRT_OK != op_ret) {
        TUYA_BTUART_ERR("uart%d init fail %d",port, op_ret);
        return OPRT_COM_ERROR;
    }

    /* 打印配置 */
    tuya_bt_uart_print_info(bt_drv->bt_uart);
    TUYA_BTUART_INFO("uart%d drv init ...", port);
    bt_drv->is_init = TRUE;

    return OPRT_OK;
}

/**
 * @brief 互斥锁
 *
 * @return STATIC
 */
STATIC VOID tuya_bt_uart_lock(VOID)
{
    if (!bt_drv->mutex) {
        return;
    }
    tuya_hal_mutex_lock(bt_drv->mutex);
}

/**
 * @brief 互斥锁
 *
 * @return STATIC
 */
STATIC VOID tuya_bt_uart_unlock(VOID)
{
    if (!bt_drv->mutex) {
        return;
    }
    tuya_hal_mutex_unlock(bt_drv->mutex);
}

/**
 * @brief 发送接口
 *
 * @param data
 * @param len
 * @return OPERATE_RET
 */
OPERATE_RET tuya_bt_uart_write(UCHAR_T *data,UINT16_T len)
{
    if (!bt_drv) {
        TUYA_BTUART_ERR("uart is not init!!!");
        return OPRT_COM_ERROR;
    }
    if (!bt_drv->is_init) {
        TUYA_BTUART_ERR("uart%d is not init!!!", bt_drv->bt_uart_port);
        return OPRT_COM_ERROR;
    }
    tuya_bt_uart_lock();
    tuya_uart_write(bt_drv->bt_uart, data, len);
    tuya_bt_uart_unlock();
    return OPRT_OK;
}

/**
 * @brief 接收接口
 *
 * @param data
 * @param len
 * @return OPERATE_RET
 */
INT_T tuya_bt_uart_read(UCHAR_T *data, UINT16_T len)
{
    if (!bt_drv) {
        TUYA_BTUART_ERR("uart is not init!!!");
        return OPRT_COM_ERROR;
    }
    if (!bt_drv->is_init) {
        TUYA_BTUART_ERR("uart%d is not init!!!", bt_drv->bt_uart);
        return OPRT_COM_ERROR;
    }
    return tuya_uart_read(bt_drv->bt_uart, data, len);
}


INT_T tuya_bt_uart_timeout(UCHAR_T *data, UINT16_T len,UINT16_T timeout)
{
    if (!bt_drv) {
        TUYA_BTUART_ERR("uart is not init!!!");
        return OPRT_COM_ERROR;
    }
    if (!bt_drv->is_init) {
        TUYA_BTUART_ERR("uart%d is not init!!!", bt_drv->bt_uart);
        return OPRT_COM_ERROR;
    }
    return tuya_uart_read_timeout(bt_drv->bt_uart, data, len,timeout);
}

OPERATE_RET tuya_bt_uart_open(tuya_uart_port_t port,INT_T baudrate)
{
    OPERATE_RET op_ret = OPRT_OK;
    tuya_uart_cfg_t uart_cfg = {
        .baudrate   = baudrate,
        .flag       = TUYA_DRV_NONBLOCK_FLAG,
        .bufsz      = UART_RECV_BUF_MAX,
        .databits   = TUYA_UART_DATA_BIT8,
        .stopbits   = TUYA_UART_STOP_BIT1,
        .parity     = TUYA_UART_PARITY_NONE
    };
    op_ret = tuya_bt_uart_init(port, &uart_cfg);
    if (OPRT_OK != op_ret) {
        TUYA_BTUART_ERR("Failed to open uart,ret=%d", op_ret);
        return op_ret;
    }
    return op_ret;
}
