 /*============================================================================
 *                                                                            *
 * Copyright (C) by Tuya Inc                                                  *
 * All rights reserved                                                        *
 *                                                                            *
 * @author  :   Linch                                                         *
 * @date    :   2019-08-28                                                    *
 * @brief   :                                                                 *
 *                                                                            *
 =============================================================================*/

/*============================ INCLUDES ======================================*/
#include <string.h>
#include "tuya_uart.h"
#include "stdio.h"
#include <termios.h>
#include "mem_pool.h"
#include <pthread.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include "uni_log.h"

/*============================ MACROS ========================================*/
#define UART_DEV_NUM            2

#define UART_DEV_CHECK(__UART, __PORT)                                      \
    __UART = uart_dev_instance_get(__PORT);                                 \
    if (NULL == __UART) {                                                   \
        return OPRT_INVALID_PARM;                                           \
    }

/*============================ TYPES =========================================*/
typedef struct {
    tuya_uart_t         dev;
    pthread_t           id;
    int                 fd;
    tuya_uart_isr_cb    isr_cb;
    int                 exit;
} uart_dev_t;

/*============================ PROTOTYPES ====================================*/
static int uart_dev_init        (tuya_uart_t *uart, tuya_uart_cfg_t *cfg);
static int uart_dev_write_byte  (tuya_uart_t *uart, uint8_t byte);
static int uart_dev_read_byte   (tuya_uart_t *uart, uint8_t *byte);
static int uart_dev_control     (tuya_uart_t *uart, uint8_t cmd, void *arg);
static int uart_dev_deinit      (tuya_uart_t *uart);

/*============================ LOCAL VARIABLES ===============================*/
static uart_dev_t s_uart_dev[UART_DEV_NUM];

static const tuya_uart_ops_t  uart_dev_ops = {
    .init       = uart_dev_init,
    .read_byte  = uart_dev_read_byte,
    .write_byte = uart_dev_write_byte,
    .control    = uart_dev_control,
    .deinit     = uart_dev_deinit,
};

static char uart_dev_path[128];

/*============================ GLOBAL VARIABLES ==============================*/
/*============================ IMPLEMENTATION ================================*/
int tuya_ipc_platform_uart_init(CHAR_T* uart_path)
{
    printf("begin");
    strncpy(uart_dev_path, uart_path, sizeof(uart_dev_path));

    s_uart_dev[TUYA_UART0].dev.ops = (tuya_uart_ops_t *)&uart_dev_ops;
    TUYA_UART_8N1_CFG(&s_uart_dev[TUYA_UART0].dev, TUYA_UART_BAUDRATE_115200, 256, 0);
    tuya_driver_register(&s_uart_dev[TUYA_UART0].dev.node, TUYA_DRV_UART, TUYA_UART0);
    printf("end");
	return OPRT_OK;
}


static void *uart_dev_irq_handler(void *parameter) //Modified by ls static void uart_dev_irq_handler
{
    uart_dev_t *uart_dev = (uart_dev_t *)parameter;
    printf("uart irq handler start: %d", pthread_self());

    for (;;) {
        if (uart_dev->exit){
            break;
        }
        fd_set readfd;
        struct timeval tv;
        tv.tv_sec = 0;
        tv.tv_usec = 100 * 1000;

        FD_ZERO(&readfd);
        FD_SET(uart_dev->fd, &readfd);
        int num = select(uart_dev->fd + 1, &readfd, NULL, NULL, &tv);
        if (num > 0 && FD_ISSET(uart_dev->fd, &readfd)){  		
            uart_dev->isr_cb(&uart_dev->dev, TUYA_UART_INT_RX_EVENT);
        }
    }

    printf("uart irq handler exit: %d", pthread_self());
	return NULL;
}

static int uart_dev_init(tuya_uart_t *uart, tuya_uart_cfg_t *cfg)
{
    printf("init begin");
    uart_dev_t *uart_dev = (uart_dev_t *)uart;

    //struct termios term_orig;
    //struct termios term_vi;

    if (TUYA_UART0 == uart->node.port) {
        struct termios term_orig;
        struct termios term_vi;
		speed_t baud;

        uart_dev->fd = open(uart_dev_path, O_RDWR);
        if (0 > uart_dev->fd) {
            printf("open failed: %s", uart_dev_path);
            return OPRT_COM_ERROR;
        }
		
		switch(cfg->baudrate) {
            case TUYA_UART_BAUDRATE_300: baud = B300; break;
            case TUYA_UART_BAUDRATE_600: baud = B600; break;
            case TUYA_UART_BAUDRATE_1200: baud = B1200; break;
            case TUYA_UART_BAUDRATE_2400: baud = B2400; break;
            case TUYA_UART_BAUDRATE_4800: baud = B4800; break;
            case TUYA_UART_BAUDRATE_9600: baud = B9600; break;
            case TUYA_UART_BAUDRATE_19200: baud = B19200; break;
            case TUYA_UART_BAUDRATE_38400: baud = B38400; break;
            case TUYA_UART_BAUDRATE_57600: baud = B57600; break;
            case TUYA_UART_BAUDRATE_115200: baud = B115200; break;
            case TUYA_UART_BAUDRATE_230400: baud = B230400; break;
            case TUYA_UART_BAUDRATE_460800: baud = B460800; break;
            case TUYA_UART_BAUDRATE_921600: baud = B921600; break;
            case TUYA_UART_BAUDRATE_1500000: baud = B1500000; break;
            default: baud = B115200; break;
        }
		
        tcgetattr(uart_dev->fd, &term_orig);
        term_vi = term_orig;
		//close by lhs
		if(0) {//if (0 != cfsetispeed(&term_vi, baud)) {
            printf("set ispeed failed: baud(%d)", baud);
            return -1;
        }
        if(0) {//if (0 != cfsetospeed(&term_vi, baud)) {
            printf("set ospeed failed: baud(%d)", baud);
            return -1;
        }
		
		term_vi.c_cflag |= (CLOCAL | CREAD);
        if (cfg->parity == TUYA_UART_PARITY_NONE) {
            term_vi.c_cflag &= ~PARENB;
        }
        if (cfg->stopbits == TUYA_UART_STOP_BIT1) {
            term_vi.c_cflag &= ~CSTOPB;
        } else {
            term_vi.c_cflag |= CSTOPB;
        }
        term_vi.c_cflag &= ~CSIZE;
        switch (cfg->databits) {
        case TUYA_UART_DATA_BIT5:
            term_vi.c_cflag |= CS5;
            break;
        case TUYA_UART_DATA_BIT6:
            term_vi.c_cflag |= CS6;
            break;
        case TUYA_UART_DATA_BIT7:
            term_vi.c_cflag |= CS7;
            break;
        case TUYA_UART_DATA_BIT8:
            term_vi.c_cflag |= CS8;
            break;
        default:
            term_vi.c_cflag |= CS8;
            break;
        }
        term_vi.c_lflag &= ~(ECHO | ECHOE | ISIG | ICANON);
        // term_vi.c_cflag &= ~CRTSCTS;

        /**
         * AT is going to use a binary protocol, so make sure to
         * turn off any CR/LF translation and the like.
         */
        term_vi.c_iflag &= ~(IXON | IXOFF | IXANY | INLCR | ICRNL);

        term_vi.c_oflag &= ~OPOST;
        term_vi.c_cc[VMIN]  = 0;
        term_vi.c_cc[VTIME] = 0;
		
        tcsetattr(uart_dev->fd, TCSANOW, &term_vi);
    }

    uart_dev->exit = 0;
    pthread_create(&uart_dev->id ,NULL, uart_dev_irq_handler, uart);
    printf("init end");

    return OPRT_OK;
}

static int uart_dev_write_byte(tuya_uart_t *uart, uint8_t byte)
{
    uart_dev_t *uart_dev = (uart_dev_t *)uart;

    write(uart_dev->fd, &byte, 1);
    printf("uart tx: %x\n", byte);

    return OPRT_OK;
}

static int uart_dev_control(tuya_uart_t *uart, uint8_t cmd, void *arg)
{
    int result = OPRT_OK;

    uart_dev_t *uart_dev = (uart_dev_t *)uart;

    switch (cmd) {
    case TUYA_DRV_SET_INT_CMD:
        break;

    case TUYA_DRV_CLR_INT_CMD:
        break;

    case TUYA_DRV_SET_ISR_CMD:
        uart_dev->isr_cb = (tuya_uart_isr_cb)arg;
        break;
    }

    return result;
}

static int uart_dev_read_byte(tuya_uart_t *uart, uint8_t *byte)
{
    uart_dev_t *uart_dev = (uart_dev_t *)uart;

    read(uart_dev->fd, byte, 1);

    return OPRT_OK;
}

static int uart_dev_deinit(tuya_uart_t *uart)
{
    printf("deinit begin");
    uart_dev_t *uart_dev = (uart_dev_t *)uart;
    uart_dev->exit = 1;
    pthread_join(uart_dev->id, NULL);
	if (uart_dev->fd > 0){
		close(uart_dev->fd);
	}
    printf("deinit end");

    return OPRT_OK;
}
